/*
 * @Author       : Eleanor Mao
 * @Date         : 2024-04-09 10:36:44
 * @LastEditTime : 2024-04-09 10:36:44
 *
 *
 */
import React, { ChangeEvent, FC, KeyboardEvent, useEffect, useMemo, useRef, useState } from "react";
import Grid from '@mui/material/Grid';
import Box from '@mui/material/Box';
import Button from '@mui/material/Button';
import TextField from '@mui/material/TextField';
import SendIcon from '@mui/icons-material/Send';
import List from '@mui/material/List';
import Divider from '@mui/material/Divider';
import { ChatItem } from "./ChatItem";
import { cloneDeep, uniqueId } from 'lodash';
import Typography from "@mui/material/Typography";
import { useLocation, useNavigate, useParams } from "react-router-dom";
import axios from 'axios';
import { Action, Definition, DSL2Json, Json2Preview, Json2Yml } from "./loadYml";
import RefreshIcon from '@mui/icons-material/Refresh';
import IconButton from '@mui/material/IconButton';
import Tooltip from '@mui/material/Tooltip';
import { EmailSender, EmailValue } from "./EmailSender";
import { AIBot } from "./AIBot";
import { GlipSender } from "./GlipSender";
import { ScriptBlock } from "./Script";
import { message } from 'mui-message';
import AutoAwesomeIcon from '@mui/icons-material/AutoAwesome';
import InputAdornment from "@mui/material/InputAdornment";
import Stack from '@mui/material/Stack';
import Switch from '@mui/material/Switch';
import { styled } from "@mui/material/styles";

const DefaultMessage = [
  'Hello! Please tell me what kind of workflow you want to generate!\n',
  'Currently the node types I support are: Chat, BotMessage, GlipSender, Script'].join('\n');

const DefaultMessageZh = [
  '你好(*´▽｀)ノノ\n 快告诉我你想生成什么样的工作流吧 ❗',
].join('\n');

const DefaultEditMessageZh = [
  '又见面啦(*╹▽╹*)\n 你已经生成过工作流啦，如果有需要修改的请告诉我吧 ❗',
].join('\n');

function getDefaultMessage(edit?: boolean): string {
  if (edit) {
    return DefaultEditMessageZh;
  }

  return DefaultMessageZh;
}

type PreValue = string | undefined | null
const preValueObj: { bot: PreValue; script: PreValue; email: PreValue; glip: PreValue } = {
  bot: '',
  script: '',
  email: '{"to":"","subject":"","content":"${localData.Desc}"}',
  glip: ''
};

interface Message {
  id: string;
  message: string;
  isMe: boolean;
  loading?: boolean;
}

interface ActionWithStateName extends Action {
  stateName: string;
}

const AntSwitch = styled(Switch)(({ theme }) => ({
  width: 28,
  height: 16,
  padding: 0,
  display: 'flex',
  '&:active': {
    '& .MuiSwitch-thumb': {
      width: 15,
    },
    '& .MuiSwitch-switchBase.Mui-checked': {
      transform: 'translateX(9px)',
    },
  },
  '& .MuiSwitch-switchBase': {
    padding: 2,
    '&.Mui-checked': {
      transform: 'translateX(12px)',
      color: '#fff',
      '& + .MuiSwitch-track': {
        opacity: 1,
      },
    },
  },
  '& .MuiSwitch-thumb': {
    boxShadow: '0 2px 4px 0 rgb(0 35 11 / 20%)',
    width: 12,
    height: 12,
    borderRadius: 6,
    transition: theme.transitions.create(['width'], {
      duration: 200,
    }),
  },
  '& .MuiSwitch-track': {
    borderRadius: 16 / 2,
    opacity: 1,
    boxSizing: 'border-box',
  },
}));

export const Create: FC<{ edit?: boolean }> = ({ edit }) => {
  const { id } = useParams();
  const [formJson, setFormJson] = useState({
    id: 'abc',
    description: '',
    name: 'My workflow',
    versionId: '',
    versionVisibleId: ''
  });
  const [chatList, setChatList] = useState<Message[]>([{
    id: uniqueId('r'),
    message: getDefaultMessage(edit),
    isMe: false
  }]);
  const [chatInput, setChatInput] = useState('');
  const [workflowContent, setWorkflowContent] = useState('');
  // eslint-disable-next-line no-template-curly-in-string
  const flowDefinition = useMemo<Definition | undefined>(() => DSL2Json(workflowContent), [workflowContent]);
  const previewImg = useMemo<string>(() => Json2Preview(flowDefinition), [flowDefinition]);
  const chatContentRef = useRef<HTMLUListElement>(null);
  const actions = useMemo<ActionWithStateName[]>(() => ((flowDefinition || {}).states || []).reduce<Array<Action & {
    stateName: string
  }>>((l, s) => {
    l.push(...s.actions.map(a => ({ ...a, stateName: s.name })));
    return l;
  }, []), [flowDefinition]);
  const emailSenderAction = useMemo<ActionWithStateName | undefined>(() => actions.find(a => a.actionType === "EmailSender"), [actions]);
  const botAddinAction = useMemo<ActionWithStateName | undefined>(() => actions.find(a => a.actionType === "Chat"), [actions]);
  const scriptAction = useMemo<ActionWithStateName | undefined>(() => actions.find(a => a.actionType === "Script"), [actions]);
  const glipSenderAction = useMemo<ActionWithStateName | undefined>(() => actions.find(a => a.actionType === "GlipSender"), [actions]);
  const [aibot, setAIBot] = useState('');
  const [script, setScript] = useState('');
  const [emailData, setEmailData] = useState<EmailValue>({
    to: '',
    subject: '',
    // eslint-disable-next-line no-template-curly-in-string
    content: '${localData.Desc}',
  });
  const [glipSenderData, setGlipSenderData] = useState('');
  const [isImageMode, setImageMode] = useState(true);
  const creatorId = '4034902020';
  const location = useLocation();
  const [teamLink, setTeamLink] = useState('');
  const debug = useMemo(() => {
    return (new URLSearchParams(location.search)).get('debug') === 'true';
  }, [location]);
  useEffect(() => {
    console.log(workflowContent);
    console.log(flowDefinition);
  }, [flowDefinition, workflowContent]);
  useEffect(() => {
    if (id) {
      axios.get<{
        id: string
        visibleId: string
        versionId: string
        versionVisibleId: string
        content: string
        name: string
        description: string
      }>(`/bot/workflow/${id}`).then(({ data }) => {
        setFormJson(data);
        setWorkflowContent(data.content || '');
      });
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);
  const handleChange = (e: ChangeEvent<HTMLInputElement>) => {
    setChatInput(e.target.value);
  };

  const handleSend = async () => {
    const rId = uniqueId('r');

    setChatList([...chatList, {
      id: uniqueId('m'),
      message: chatInput,
      isMe: true
    }, {
      id: rId,
      message: '',
      isMe: false,
      loading: true
    }]);
    setChatInput('');
    try {
      const { data } = await axios.post<{ answer: string; workflowContent: string }>(`/bot/workflow/${id}/messages`, {
        "question": chatInput
      });
      setWorkflowContent(data.workflowContent);
      setChatList(list => list.map(l => {
        if (l.id === rId) {
          l.loading = false;
          l.message = data.answer;
        }
        return l;
      }));
    } catch (e) {
      setChatList(list => list.filter(l => l.id !== rId));
    }
  };

  useEffect(() => {
    if (glipSenderAction) {
      let actionValueString = glipSenderAction.message;
      if (actionValueString !== preValueObj.glip && glipSenderData === preValueObj.glip) {
        setGlipSenderData(actionValueString || '');
        preValueObj.glip = actionValueString || '';
      }
    } else {
      if (preValueObj.glip) {
        preValueObj.glip = '';
        // eslint-disable-next-line no-template-curly-in-string
        setGlipSenderData('${localData.Desc}');
      }
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [glipSenderAction]);
  useEffect(() => {
    if (scriptAction) {
      if (scriptAction.scriptSource !== preValueObj.script && script === preValueObj.script) {
        setScript(scriptAction.scriptSource || '');
        preValueObj.script = scriptAction.scriptSource || '';
      }
    } else if (preValueObj.script) {
      preValueObj.script = '';
      setScript('');
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [scriptAction]);

  useEffect(() => {
    if (botAddinAction) {
      if (botAddinAction.id !== preValueObj.bot && aibot === preValueObj.bot) {
        setAIBot(botAddinAction.id || '');
        preValueObj.bot = botAddinAction.id || '';
      }
    } else if (preValueObj.bot) {
      preValueObj.bot = '';
      setAIBot('');
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [botAddinAction]);
  useEffect(() => {
    if (emailSenderAction) {
      const newValueStr = JSON.stringify({
        to: emailSenderAction.to || '',
        subject: emailSenderAction.subject || '',
        content: emailSenderAction.content || ''
      });
      const crtValueStr = JSON.stringify({
        to: emailData.to,
        subject: emailData.subject,
        content: emailData.content
      });
      if (newValueStr !== preValueObj.email && crtValueStr === preValueObj.email) {
        setEmailData({
          to: emailSenderAction.to || '',
          subject: emailSenderAction.subject || '',
          content: emailSenderAction.content || ''
        });
        preValueObj.email = newValueStr || '';
      }
    } else {
      setEmailData({
        to: '',
        subject: '',
        // eslint-disable-next-line no-template-curly-in-string
        content: '${localData.Desc}',
      });
    }
  }, [emailSenderAction]);
  useEffect(() => {
    if (chatContentRef.current) {
      chatContentRef.current.scrollTop = chatContentRef.current.scrollHeight - chatContentRef.current.offsetHeight;
    }
  }, [chatList]);

  const handleRefresh = async () => {
    await axios.delete(`/bot/workflow/${id}/messages`);
    setChatList([{
      id: uniqueId('r'),
      message: getDefaultMessage(edit),
      isMe: false
    }]);
    setChatInput('');
    setWorkflowContent('');
  };


  const navigate = useNavigate();
  const handleSubmit = async () => {
    if (!flowDefinition) return;
    const newJson = cloneDeep(flowDefinition);
    newJson.states.forEach(state => {
      state.actions.forEach(action => {
        if (action.actionType === 'Chat') {
          action.id = aibot;
        }
        if (action.actionType === 'Script') {
          action.scriptSource = script;
        }
        if (action.actionType === 'EmailSender') {
          action.to = emailData.to;
          action.subject = emailData.subject;
          action.content = emailData.content;
        }
        if (action.actionType === 'GlipSender') {
          // action.url = glipSenderData.url;
          // eslint-disable-next-line no-template-curly-in-string
          action.message = glipSenderData || '${localData.Desc}';
        }
      });
    });
    console.log(Json2Yml(newJson));
    const channelId = (teamLink.replace(/[^\d]+/, '')) + '-' + creatorId;
    console.log(channelId);
    await axios.put(`/bot/workflow/${id}`, {
      name: formJson.name,
      content: Json2Yml(newJson)
    });
    if (!debug) {
      await axios.delete(`/bot/workflow/${id}/bind/${channelId}`);
      await axios.post(`/bot/workflow/${id}/bind`, { channelId });
    }
    message.success(`Publish Success! ${debug ? 'Back to List and go bind your channel!' : ''}`);
    setTimeout(() => {
      navigate(`/${debug ? '?debug=true' : ''}`);
    }, 3000);
  };

  return (
    <>
      <Grid container spacing={2}>
        <Grid item xs={12}>
          <TextField value={formJson.name} fullWidth variant="outlined" label="Workflow Name" InputProps={{
            readOnly: true,
          }}
                     placeholder="Please name your workflow" onChange={e => {
            setFormJson(json => ({
              ...json,
              name: e.target.value
            }));
          }}/>
          {/*<Divider sx={{ mt: 4, mb: -2 }}/>*/}
        </Grid>
        <Grid item xs={12}>
          <Grid container spacing={2}>
            <Grid item xs>
              <Box sx={{
                border: 1,
                height: 600,
                borderRadius: 1,
                mt: 4,
                borderColor: 'rgba(255,255,255,0.23)',
                position: 'relative'
              }}>
                <Tooltip title="Clear AI 🧠">
                  <IconButton onClick={handleRefresh} style={{ position: 'absolute', right: 10, top: 10, zIndex: 5 }}>
                    <RefreshIcon/>
                  </IconButton>
                </Tooltip>
                <div style={{ height: '100%', display: 'flex', flexDirection: 'column' }}>
                  <List ref={chatContentRef}
                        style={{ flex: '1 1 auto', overflowY: 'auto', overflowX: 'hidden' }}
                        sx={{ p: 2 }}>
                    {chatList.map(chat => <ChatItem isMe={chat.isMe} message={chat.message} key={chat.id}
                                                    loading={chat.loading}/>)}
                  </List>
                  <Box sx={{ display: 'flex', flex: '0 0 auto', p: 2, borderTop: 1, borderColor: 'divider' }}>
                    <Grid container spacing={2}>
                      {/*<Grid item xs={12} style={{ gap: 8 }}>*/}
                      {/*  <Stack direction="row" spacing={1}>*/}
                      {/*    <Typography variant="body2">*/}
                      {/*      Supported Action Types:*/}
                      {/*    </Typography>*/}
                      {/*    {SupportedBlock.map(label => (*/}
                      {/*      <Chip label={label} key={label} size="small" clickable color="primary"*/}
                      {/*            onClick={handleClickChip(label)}/>))}*/}
                      {/*  </Stack>*/}
                      {/*</Grid>*/}
                      <Grid item xs>
                        <TextField fullWidth size="small" onChange={handleChange}
                                   onKeyDown={(e: KeyboardEvent<HTMLInputElement>) => {
                                     if (e.key === 'Enter' && chatInput.length) {
                                       e.preventDefault();
                                       handleSend();
                                     }
                                   }}
                                   autoFocus
                                   InputProps={{
                                     startAdornment: <InputAdornment position="start"><AutoAwesomeIcon color="primary"/></InputAdornment>
                                   }}
                                   variant="outlined"
                                   focused placeholder="Enter your goal ..."
                                   multiline maxRows={3}
                                   value={chatInput}/>
                      </Grid>
                      <Grid item xs="auto" justifyContent="center">
                        <Button variant="contained" size="large" disabled={!chatInput.length} onClick={handleSend}>
                          <SendIcon/>
                        </Button>
                      </Grid>
                    </Grid>
                  </Box>
                </div>
              </Box>
            </Grid>
            <Grid item xs={4}>
              <Box sx={{
                border: 1,
                height: 600,
                borderRadius: 1,
                mt: 4,
                borderColor: 'rgba(255,255,255,0.23)',
                textAlign: 'center'
              }}>
                {!previewImg ?
                  <div style={{ padding: 16 }}>👋 The flow chart will be previewed here! <br/>😻 Tell the AI robot what
                    you want to
                    do! </div> :
                  (
                    <>
                      <Stack sx={{ p: 2 }} direction="row" spacing={1} alignItems="center" justifyContent="flex-end">
                        <Typography variant="body2">Raw</Typography>
                        <AntSwitch checked={isImageMode} onChange={(_, checked) => setImageMode(checked)}/>
                        <Typography variant="body2">Image</Typography>
                      </Stack>
                      {isImageMode ?
                        <img src={previewImg} style={{ maxWidth: '100%', maxHeight: '100%' }} alt="preview"/> :
                        <pre style={{ height: 546, overflow: 'auto', marginTop: 0 }}><code style={{
                          fontSize: 12,
                          textAlign: 'left',
                          display: 'block',
                          padding: 16,
                        }}>{workflowContent}</code></pre>
                      }
                    </>
                  )
                }
              </Box>
            </Grid>
          </Grid>
          <Divider sx={{ mt: 4, mb: 4 }}/>
          {!!scriptAction || !!emailSenderAction || !!botAddinAction || !!glipSenderAction ?
            <Typography variant="subtitle2" sx={{ mb: 2 }}>
              Please fill in the following information related to your workflow nodes. 👇🏻👇🏻 </Typography> : null}
          {!!emailSenderAction && <EmailSender value={emailData} onChange={setEmailData}/>}
          {!!botAddinAction && <AIBot name={botAddinAction.stateName} value={aibot} onChange={setAIBot}/>}
          {!!scriptAction && <ScriptBlock name={scriptAction.stateName} value={script} onChange={setScript}/>}
          {!!glipSenderAction &&
            <GlipSender debug={debug} teamLink={teamLink} id={id} edit={edit} value={glipSenderData}
                        onChange={(value, isTeamLink) => {
                          if (isTeamLink) {
                            setTeamLink(value);
                          } else {
                            setGlipSenderData(value);
                          }
                        }}/>}
        </Grid>
      </Grid>
      <div style={{ height: 80 }}></div>
      <Box
        sx={{
          position: 'fixed',
          bottom: 0,
          left: 0,
          right: 0,
          p: 2,
          boxShadow: 2,
          borderTop: 1,
          zIndex: 10,
          bgcolor: 'background.default',
          borderColor: 'divider',
          display: { xs: 'flex' }, justifyContent: "flex-end"
        }}>
        <Button variant="contained" style={{ width: 100 }} onClick={handleSubmit}>Publish</Button>
      </Box>
    </>
  );
};
