/*
 * @Author       : Eleanor Mao
 * @Date         : 2024-04-09 09:38:20
 * @LastEditTime : 2024-04-09 09:38:20
 *
 *
 */
import React, { useEffect, useMemo, useState } from "react";
import Button from '@mui/material/Button';
import Box from '@mui/material/Box';
import { styled } from '@mui/material/styles';
import Typography from '@mui/material/Typography';
import CardActionArea from '@mui/material/CardActionArea';
import Switch from '@mui/material/Switch';
import { Link, useLocation, useNavigate } from "react-router-dom";
import AddIcon from '@mui/icons-material/Add';
import Card from '@mui/material/Card';
import CardActions from '@mui/material/CardActions';
import CardContent from '@mui/material/CardContent';
import axios from "axios";
import Container from '@mui/material/Container';
import Dialog from '@mui/material/Dialog';
import DialogTitle from '@mui/material/DialogTitle';
import DialogActions from '@mui/material/DialogActions';
import DialogContent from '@mui/material/DialogContent';
import DialogContentText from '@mui/material/DialogContentText';
import TextField from '@mui/material/TextField';
import { getRandomEmoji } from "./randomEmoji";
import { formatTime } from "./timeFormat";
import { message } from "mui-message";

const Flex = styled(Box)(() => ({
  display: 'flex',
  flexWrap: 'wrap'
}));

interface Workflow {
  description: string;
  id: string;
  name: string;
  lastModifiedTime: string;
  emoji?: string;
}

export const List = () => {
  const [list, setList] = useState<Workflow[]>([]);
  const [showModal, setShowModal] = useState(false);
  const [showBindModal, setShowBindModel] = useState('');

  const navigate = useNavigate();
  useEffect(() => {
    axios.get<{
      total: number
      workflows: Workflow[]
    }>('/api/public/wf/config/v1/account/10086/workflows').then(({ data }) => {
      setList(data.workflows.map(it => ({
        ...it,
        emoji: getRandomEmoji()
      })));
    });
  }, []);
  const handleOpen = () => {
    setShowModal(true);
  };
  const handleOpenBind = (id: string) => {
    setShowBindModel(id);
  };
  const handleClose = () => {
    setShowModal(false);
  };
  const handleCloseBindModel = () => {
    setShowBindModel('');
  };
  const location = useLocation();
  const debug = useMemo(() => {
    return (new URLSearchParams(location.search)).get('debug') === 'true';
  }, [location]);
  const handleSubmit = (event: React.FormEvent<HTMLFormElement>) => {
    event.preventDefault();
    const formData = new FormData(event.currentTarget);
    const formJson = Object.fromEntries((formData as any).entries());
    axios.post('/bot/workflow', { name: formJson.name }).then(({ data }) => {
      handleClose();
      navigate(`/create/${data.visibleId}${debug ? '?debug=true' : ''}`,);
    });
  };
  const handleSubmitBind = async (event: React.FormEvent<HTMLFormElement>) => {
    event.preventDefault();
    const formData = new FormData(event.currentTarget);
    const formJson = Object.fromEntries((formData as any).entries());
    const channelId = `${formJson.groupId.trim()}-${formJson.creatorId.trim()}`;
    await axios.delete(`/bot/workflow/${showBindModal}/bind/${channelId}`);
    await axios.post(`/bot/workflow/${showBindModal}/bind`, { channelId });
    message.success('Success!');
    handleCloseBindModel();
  };
  return (
    <Container maxWidth="lg">
      <Flex style={{ justifyContent: 'flex-end' }}>
        <Button startIcon={<AddIcon/>} variant="contained" onClick={handleOpen}>Create Workflow</Button>
      </Flex>
      <Flex sx={{ gap: 4, mt: 4 }}>
        {list.map(item => (
          <Link
            key={item.id}
            to={`/detail/${item.id}${debug ? '?debug=true' : ''}`}
          >
            <Card sx={{ width: 263, borderRadius: 2, boxShadow: 2 }}>
              <CardActionArea>
                <CardContent style={{ height: 100 }}>
                  <Typography gutterBottom variant="h5" component="div">
                    {item.emoji} {item.name || 'My Workflow'}
                  </Typography>
                </CardContent>
                <CardActions
                  sx={{
                    display: { xs: 'flex' },
                    justifyContent: "space-between",
                    borderTop: 1,
                    borderColor: 'divider'
                  }}>
                  <Typography component="span" variant="caption" style={{ opacity: .6 }}>Last
                    modified: {formatTime(item.lastModifiedTime)}</Typography>
                  {debug ?
                    <Button size="small" onClick={(e: any) => {
                      e.preventDefault();
                      handleOpenBind(item.id);
                    }}>Bind</Button> : <Button size="small">Copy</Button>}
                  {/*<Switch defaultChecked/>*/}
                </CardActions>
              </CardActionArea>
            </Card>
          </Link>))}
      </Flex>
      <Dialog
        open={showModal}
        onClose={handleClose}
        PaperProps={{
          component: 'form',
          onSubmit: handleSubmit,
        }}
      >
        <DialogTitle>Create Workflow</DialogTitle>
        <DialogContent>
          <DialogContentText>
            To create a workflow, please name your workflow first.
          </DialogContentText>
          <TextField
            autoFocus
            required
            margin="dense"
            id="name"
            name="name"
            label="Workflow Name"
            fullWidth
            focused
            variant="outlined"
          />
        </DialogContent>
        <DialogActions>
          <Button onClick={handleClose}>Cancel</Button>
          <Button type="submit">Create</Button>
        </DialogActions>
      </Dialog>
      <Dialog open={!!showBindModal} onClose={handleCloseBindModel}
              PaperProps={{
                component: 'form',
                onSubmit: handleSubmitBind,
              }}>
        <DialogTitle>Bind Channel</DialogTitle>
        <DialogContent>
          <TextField
            required
            margin="dense"
            id="groupId" focused
            variant="outlined"
            label="Group Id" fullWidth name="groupId"
            helperText="You can get team groupId from team link"
          />
          <TextField
            required
            margin="dense"
            id="creatorId" focused
            variant="outlined"
            label="Creator Id" fullWidth name="creatorId"
          />
        </DialogContent>
        <DialogActions>
          <Button onClick={handleCloseBindModel}>Cancel</Button>
          <Button type="submit">Bind</Button>
        </DialogActions>
      </Dialog>
    </Container>
  );
};
